USD = 'usd'
BTC = 'btc'
BCH = 'bch'
BTG = 'btg'
ETH = 'eth'
ETC = 'etc'
LTC = 'ltc'
IOT = 'iot'
BFX = 'bfx'
XRP = 'xrp'
EOS = 'eos'

SYMBOLS = [
    USD,
    BTC,
    BCH,
    BTG,
    ETH,
    ETC,
    LTC,
    IOT,
    BFX,
    XRP,
    EOS,
]
SYMBOL_VARIANTS = {
    USD: ('zusd', ),
    BTC: ('xxbt', ),
    BCH: ('bcc', ),
    BTG: (),
    ETH: ('xeth', ),
    ETC: (),
    LTC: ('xltc', ),
    IOT: (),
    BFX: (),
    XRP: (),
    EOS: (),
}

BTC_USD = '{}_{}'.format(BTC, USD)
ETH_USD = '{}_{}'.format(ETH, USD)
ETC_USD = '{}_{}'.format(ETC, USD)
LTC_USD = '{}_{}'.format(LTC, USD)
BCH_USD = '{}_{}'.format(BCH, USD)
XRP_USD = '{}_{}'.format(XRP, USD)
EOS_USD = '{}_{}'.format(EOS, USD)
BTG_USD = '{}_{}'.format(BTG, USD)

SYMBOL_PAIRS = [
    BTC_USD,
    ETH_USD,
    ETC_USD,
    LTC_USD,
    BCH_USD,
    XRP_USD,
    EOS_USD,
    BTG_USD,
]
SYMBOL_PAIR_VARIANTS = {
    BTC_USD: ('btcusd', 'xbtusd', 'xxbtzusd', 'btc0929', 'btc1229', ),
    ETH_USD: ('ethusd', 'eth0511', 'eth0518', 'eth0629', 'xethzusd', ),
    LTC_USD: ('ltcusd', 'ltc0511', 'ltc0518', 'ltc0629'),
    BCH_USD: ('bchusd', 'bccusd'),
    XRP_USD: ('xrpusd', ),
    EOS_USD: ('eosusd', ),
    BTG_USD: ('btgusd', ),
    ETC_USD: ('etcusd', ),
}



from tests import BaseXchangeTestCase
from xchange.factories import ExchangeClientFactory
from xchange.constants import exchanges
from xchange.clients import BitfinexClient


class ClientFactoryTestCase(BaseXchangeTestCase):

    def test_create_client_class(self):
        ClientClass = ExchangeClientFactory.get_client(exchanges.BITFINEX)
        self.assertEqual(ClientClass, BitfinexClient)

    def test_create_client_not_implemented(self):
        exchange_name = 'wontwork'
        error_msg = 'Client for exchange "{}" is not implemented'.format(exchange_name)
        with self.assertRaisesRegexp(NotImplementedError, error_msg):
            ExchangeClientFactory.get_client(exchange_name)



import time
import hmac
import json
import base64
import hashlib
from decimal import Decimal

from xchange import exceptions
from xchange.constants import currencies, exchanges
from xchange.clients.base import BaseExchangeClient
from xchange.validators import is_restricted_to_values, is_instance, passes_test
from xchange.models.bitfinex import (
    BitfinexOrderBook, BitfinexAccountBalance, BitfinexOrder, BitfinexTicker,
    BitfinexPosition)


class BitfinexClient(BaseExchangeClient):
    BASE_API_URL = 'https://api.bitfinex.com'
    ERROR_CLASS = exceptions.BitfinexException
    SYMBOLS_MAPPING = {
        currencies.BTC_USD: 'btcusd',
        currencies.ETH_USD: 'ethusd',
        currencies.ETC_USD: 'etcusd',
        currencies.LTC_USD: 'ltcusd',
        currencies.BCH_USD: 'babusd', # Bitcoin Cash ABC
        currencies.XRP_USD: 'xrpusd',
        currencies.EOS_USD: 'eosusd',
        currencies.BTG_USD: 'btgusd',
    }

    def _sign_payload(self, payload):
        payload_dump = json.dumps(payload)
        data = base64.standard_b64encode(payload_dump.encode('utf8'))
        hash = hmac.new(self.api_secret.encode('utf8'), data, hashlib.sha384)
        signature = hash.hexdigest()
        return {
            "X-BFX-APIKEY": self.api_key,
            "X-BFX-SIGNATURE": signature,
            "X-BFX-PAYLOAD": data
        }

    # transformation functions

    def _transform_account_balance(self, json_response):
        """
        Original JSON response:
        [
            {'amount': '0.0', 'available': '0.0', 'currency': 'btc', 'type': 'deposit'},
            {'amount': '0.0', 'available': '0.0', 'currency': 'btc', 'type': 'exchange'},
            {'amount': '0.0', 'available': '0.0', 'currency': 'btc', 'type': 'trading'},
            {'amount': '0.0', 'available': '0.0', 'currency': 'iot', 'type': 'deposit'},
            ...
        ]
        """
        # only take care of trading balances
        return [doc for doc in json_response if doc['type'] == 'trading']

    # public endpoints

    def get_ticker(self, symbol_pair, **kwargs):
        is_restricted_to_values(symbol_pair, currencies.SYMBOL_PAIRS)

        symbol_pair = self.SYMBOLS_MAPPING[symbol_pair]
        return self._get('/v1/pubticker/{}'.format(symbol_pair),
                         model_class=BitfinexTicker, **kwargs)

    def get_order_book(self, symbol_pair, **kwargs):
        is_restricted_to_values(symbol_pair, currencies.SYMBOL_PAIRS)

        symbol_pair = self.SYMBOLS_MAPPING[symbol_pair]
        return self._get('/v1/book/{}'.format(symbol_pair),
                         model_class=BitfinexOrderBook, **kwargs)

    # authenticated endpoints

    def get_account_balance(self, symbol=None, **kwargs):
        is_restricted_to_values(symbol, currencies.SYMBOLS + [None])

        path = '/v1/balances'
        payload = {
            'request': path,
            'nonce': str(time.time())
        }
        signed_payload = self._sign_payload(payload)
        data = self._post(path, headers=signed_payload,
                          transformation=self._transform_account_balance,
                          model_class=BitfinexAccountBalance, **kwargs)
        if symbol is None:
            return data
        for symbol_balance in data:
            if symbol_balance.symbol == symbol:
                return symbol_balance
        return self._empty_account_balance(symbol)

    def get_open_orders(self, symbol_pair, **kwargs):
        is_restricted_to_values(symbol_pair, currencies.SYMBOL_PAIRS)

        path = '/v1/orders'
        payload = {
            'request': path,
            'nonce': str(time.time())
        }
        signed_payload = self._sign_payload(payload)
        data = self._post(path, headers=signed_payload,
                          model_class=BitfinexOrder, **kwargs)
        return [order for order in data
                if order['symbol_pair'] == symbol_pair]

    def open_order(self, action, amount, symbol_pair,
                   price, order_type, **kwargs):
        """
        Creates a new Order.

        :action:
            exchanges.ACTIONS choice
        :amount:
            Decimal, float, integer or string representing number value.
        :symbol_pair:
            currencies.SYMBOL_PAIRS choice
        :price:
            Decimal, float, integer or string representing number value.
        :order_type:
            exchanges.ORDER_TYPES choice
        """
        # validate arguments
        is_restricted_to_values(action, exchanges.ACTIONS)

        is_instance(amount, (Decimal, float, int, str))
        passes_test(amount, lambda x: Decimal(x))

        is_restricted_to_values(symbol_pair, currencies.SYMBOL_PAIRS)

        is_instance(price, (Decimal, float, int, str))
        passes_test(price, lambda x: isinstance(Decimal(x), Decimal))

        is_restricted_to_values(order_type, exchanges.ORDER_TYPES)

        path = '/v1/order/new'
        symbol_pair = self.SYMBOLS_MAPPING[symbol_pair]
        payload = {
            'request': path,
            'nonce': str(time.time()),
            'side': action,
            'amount': str(amount),
            'symbol': symbol_pair,
            'price': str(price),
            'type': order_type,
        }
        signed_payload = self._sign_payload(payload)
        return self._post(path, headers=signed_payload,
                          model_class=BitfinexOrder, **kwargs)

    def cancel_order(self, order_id, **kwargs):
        path = '/v1/order/cancel'
        order_id = int(order_id)
        payload = {
            'request': path,
            'nonce': str(time.time()),
            'order_id': order_id
        }
        signed_payload = self._sign_payload(payload)
        return self._post(path, headers=signed_payload, **kwargs)

    def cancel_all_orders(self, symbol_pair, **kwargs):
        is_restricted_to_values(symbol_pair, currencies.SYMBOL_PAIRS)

        path = '/v1/order/cancel/multi'
        orders = self.get_open_orders(symbol_pair=symbol_pair)
        if not orders:
            return
        payload = {
            'request': path,
            'nonce': str(time.time()),
            'order_ids': [order.id for order in orders]
        }
        signed_payload = self._sign_payload(payload)
        return self._post(path, headers=signed_payload, **kwargs)

    def get_open_positions(self, symbol_pair, **kwargs):
        is_restricted_to_values(symbol_pair, currencies.SYMBOL_PAIRS)

        path = '/v1/positions'
        payload = {
            'request': path,
            'nonce': str(time.time()),
        }
        signed_payload = self._sign_payload(payload)
        positions = self._post(path, headers=signed_payload,
                               model_class=BitfinexPosition, **kwargs)
        return [pos for pos in positions
                if pos['symbol_pair'] == symbol_pair]

    def close_position(self, position_id, symbol_pair, **kwargs):
        is_restricted_to_values(symbol_pair, currencies.SYMBOL_PAIRS)

        positions = self.get_open_positions(symbol_pair)
        try:
            pos = [pos for pos in positions if pos.id == position_id][0]
        except IndexError:
            raise self.ERROR_CLASS('Could not find position with '
                                   'ID: "{}"'.format(position_id))

        # as we want to close the position,
        # we need to performe the opposite action to the given one.
        action = exchanges.SELL if pos.action == exchanges.BUY else exchanges.BUY

        return self.open_order(
            action, pos.amount, pos.symbol_pair, pos.price,
            exchanges.MARKET, **kwargs)

    def close_all_positions(self, symbol_pair, **kwargs):
        is_restricted_to_values(symbol_pair, currencies.SYMBOL_PAIRS)

        positions = self.get_open_positions(symbol_pair=symbol_pair)
        for pos in positions:
            self.close_position(pos.id, pos.symbol_pair, **kwargs)



from decimal import Decimal

from xchange import exceptions
from tests import BaseXchangeTestCase
from xchange.constants import exchanges
from xchange.models.base import OrderBook
from xchange.utils import volume_weighted_average_price, worst_order_price


class VolumeWeightedAveragePriceTestCase(BaseXchangeTestCase):

    def setUp(self):
        self.order_book = OrderBook({
            "asks": [
                # (price_in_btc, amount_in_btc),
                (Decimal('9000'), Decimal('0.1')),
                (Decimal('8000'), Decimal('0.4')),
                (Decimal('7000'), Decimal('0.3')),
            ],
            "bids": [
                # (price_in_btc, amount_in_btc),
                (Decimal('3000'), Decimal('0.3')),
                (Decimal('2000'), Decimal('0.4')),
                (Decimal('1000'), Decimal('0.1')),
            ]
        })

    def test_volume_weighted_average_price_buy_order(self):
        avg_price = volume_weighted_average_price(
            exchanges.BUY,
            self.order_book,
            Decimal('0.5')
        )
        # (7000 * 0.3 + 8000 * 0.2) / 0.5 = 7400
        self.assertEqual(avg_price, Decimal('7400'))

    def test_volume_weighted_average_price_sell_order(self):
        avg_price = volume_weighted_average_price(
            exchanges.SELL,
            self.order_book,
            Decimal('0.5')
        )
        # (3000 * 0.3 + 2000 * 0.2) / 0.5 = 2600
        self.assertEqual(avg_price, Decimal('2600'))

    def test_volume_weighted_average_price_no_market_depth(self):
        with self.assertRaisesRegexp(exceptions.InsufficientMarketDepth,
                                     'Not enough depth in OrderBook to sell 2.0 volume'):
            volume_weighted_average_price(
                exchanges.SELL,
                self.order_book,
                Decimal('2.0')
            )


class WorstOrderPriceTestCase(BaseXchangeTestCase):

    def setUp(self):
        self.order_book = OrderBook({
            "asks": [
                # (price_in_btc, amount_in_btc),
                (Decimal('9000'), Decimal('0.1')),
                (Decimal('8000'), Decimal('0.4')),
                (Decimal('7000'), Decimal('0.3')),
            ],
            "bids": [
                # (price_in_btc, amount_in_btc),
                (Decimal('3000'), Decimal('0.3')),
                (Decimal('2000'), Decimal('0.4')),
                (Decimal('1000'), Decimal('0.1')),
            ]
        })

    def test_worst_order_price_buy_order(self):
        worst_price = worst_order_price(
            exchanges.BUY,
            self.order_book,
            Decimal('0.5')
        )
        self.assertEqual(worst_price, Decimal('8000'))

    def test_worst_order_price_buy_order_limit_amount(self):
        worst_price = worst_order_price(
            exchanges.BUY,
            self.order_book,
            Decimal('0.3')
        )
        self.assertEqual(worst_price, Decimal('7000'))

    def test_worst_order_price_sell_order(self):
        worst_price = worst_order_price(
            exchanges.SELL,
            self.order_book,
            Decimal('0.5')
        )
        self.assertEqual(worst_price, Decimal('2000'))

    def test_worst_order_price_sell_order_limit_amount(self):
        worst_price = worst_order_price(
            exchanges.SELL,
            self.order_book,
            Decimal('0.3')
        )
        self.assertEqual(worst_price, Decimal('3000'))

    def test_worst_order_price_no_market_depth(self):
        with self.assertRaisesRegexp(exceptions.InsufficientMarketDepth,
                                     'Not enough depth in OrderBook to sell 2.0 volume'):
            worst_order_price(
                exchanges.SELL,
                self.order_book,
                Decimal('2.0')
            )



import time
import hmac
import base64
import hashlib
from decimal import Decimal
try:
    from urllib.parse import urlencode
except ImportError:
     from urllib import urlencode

from xchange import exceptions
from xchange.constants import exchanges, currencies
from xchange.clients.base import BaseExchangeClient
from xchange.validators import is_restricted_to_values, is_instance, passes_test
from xchange.models.kraken import (
    KrakenOrderBook, KrakenAccountBalance, KrakenOrder, KrakenTicker,
    KrakenPosition)


class KrakenClient(BaseExchangeClient):
    USERREF = 7886259  # random number
    BASE_API_URL = 'https://api.kraken.com'
    ERROR_CLASS = exceptions.KrakenException
    SYMBOLS_MAPPING = {
        currencies.BTC_USD: 'XBTUSD',
        currencies.ETH_USD: 'ETHUSD',
        currencies.ETC_USD: 'ETCUSD',
        currencies.LTC_USD: 'LTCUSD',

        currencies.BCH_USD: 'BCHUSD',
        currencies.XRP_USD: 'XRPUSD',
        currencies.EOS_USD: 'EOSUSD',
    }

    def _sign_payload(self, urlpath, payload):
        postdata = urlencode(payload)
        encoded = (str(payload['nonce']) + postdata).encode()
        message = urlpath.encode() + hashlib.sha256(encoded).digest()
        signature = hmac.new(base64.b64decode(self.api_secret),
                             message, hashlib.sha512)
        sigdigest = base64.b64encode(signature.digest())
        return sigdigest.decode()

    def get_userref(self, symbol_pair):
        """
        TODO: Explain this
        """
        index = sorted(self.SYMBOLS_MAPPING).index(symbol_pair)
        return int('{}{}'.format(self.USERREF, index))

    # transformation functions

    def _transform_ticker(self, json_response):
        """
        Original JSON reponse:
        {'error': [],
         'result': {'XXBTZUSD': {
             'a': ['3809.00000', '1', '1.000'],
             'b': ['3803.60000', '1', '1.000'],
             'c': ['3809.30000', '0.08000000'],
             'h': ['3830.30000', '3830.30000'],
             'l': ['3570.10000', '3525.10000'],
             'o': '3606.80000',
             'p': ['3717.00933', '3650.22371'],
             't': [6565, 14224],
             'v': ['2204.84583619', '4523.49526430']}}}
        """
        for symbol, ticker in json_response['result'].items():
            # as we will always get only one ticker dict, return the first
            # one in the iteration.
            return ticker

    def _transform_account_balance(self, json_response):
        """
        Original JSON response:
        {'error': [], 'result': {'XXBT': '0.0142771914', 'ZUSD': '0.0000'}}
        """
        return [
            {'currency': key, 'amount': value}
            for key, value in json_response['result'].items()
        ]

    def _transform_new_order(self, json_response):
        """
        Original JSON response:
        {'error': [],
         'result': {'descr': {'order': 'sell 0.02000000 ETHUSD @ limit 900.00 with 2:1 '
                                       'leverage'},
                    'txid': ['ODF2C3-OVVBA-HAYTEN']}}
        """
        return {
            'id': json_response['result']['txid'][0]
        }

    def _transform_open_orders(self, json_response):
        """
        Original JSON response:
        {
            'error': [],
            'result': {
                'open': {
                    'OGYUJ3-LSWJV-4OD4DU': {
                        'cost': '0.00000',
                        'descr': {
                            'leverage': 'none',
                            'order': 'sell 0.00500000 XBTUSD @ limit 5000.0',
                            'ordertype': 'limit',
                            'pair': 'XBTUSD',
                            'price': '5000.0',
                            'price2': '0',
                            'type': 'sell'
                        },
                        'expiretm': 0,
                        'fee': '0.00000',
                        'misc': '',
                        'oflags': 'fciq',
                        'opentm': 1505603660.2901,
                        'price': '0.00000',
                        'refid': None,
                        'starttm': 0,
                        'status': 'open',
                        'userref': None,
                        'vol': '0.00500000',
                        'vol_exec': '0.00000000'
                    }
                }
            }
        }
        """
        transformed_response = []
        for status, orders_dict in json_response['result'].items():
            for order_id, order_data in orders_dict.items():
                order_data['id'] = order_id
                order_data['status'] = status
                transformed_response.append(order_data)
        return transformed_response

    def _transform_open_positions(self, json_response):
        """
        Original JSON response:
        {
            'error': [],
            'result': {
                'TZNYBD-GOE2N-4LTHWQ': {
                    'cost': '18.67650',
                    'fee': '0.05043',
                    'margin': '9.33825',
                    'misc': '',
                    'net': '+0.0352',
                    'oflags': '',
                    'ordertxid': 'OJF4YH-EPNOR-PKWMGG',
                    'ordertype': 'market',
                    'pair': 'XXBTZUSD',
                    'posstatus': 'open',
                    'rollovertm': '1505618241',
                    'terms': '0.0100% per 4 hours',
                    'time': 1505603841.1963,
                    'type': 'sell',
                    'value': '18.6',
                    'vol': '0.00500000',
                    'vol_closed': '0.00000000'
                }
            }
        }
        """
        transformed_response = []
        for position_id, position_dict in json_response['result'].items():
            position_dict['id'] = position_id
            transformed_response.append(position_dict)
        return transformed_response

    # public endpoints

    def get_ticker(self, symbol_pair):
        is_restricted_to_values(symbol_pair, currencies.SYMBOL_PAIRS)

        symbol_pair = self.SYMBOLS_MAPPING[symbol_pair]
        params = {'pair': symbol_pair}
        return self._get('/0/public/Ticker', params=params,
                         transformation=self._transform_ticker,
                         model_class=KrakenTicker)

    def get_order_book(self, symbol_pair):
        is_restricted_to_values(symbol_pair, currencies.SYMBOL_PAIRS)

        symbol_pair = self.SYMBOLS_MAPPING[symbol_pair]
        params = {'pair': symbol_pair}
        return self._get('/0/public/Depth', params=params,
                         model_class=KrakenOrderBook)

    # authenticated endpoints

    def get_account_balance(self, symbol=None):
        is_restricted_to_values(symbol, currencies.SYMBOLS + [None])

        path = '/0/private/Balance'
        payload = {
            'nonce': int(1000 * time.time()),
        }
        headers = {
            'API-Key': self.api_key,
            'API-Sign': self._sign_payload(path, payload)
        }
        data = self._post(path, headers=headers, body=payload,
                          transformation=self._transform_account_balance,
                          model_class=KrakenAccountBalance)

        if symbol is None:
            return data
        for symbol_balance in data:
            if symbol_balance.symbol == symbol:
                return symbol_balance
        return self._empty_account_balance(symbol)

    def get_open_orders(self, symbol_pair):
        is_restricted_to_values(symbol_pair, currencies.SYMBOL_PAIRS)

        path = '/0/private/OpenOrders'
        payload = {
            'nonce': int(1000 * time.time()),
        }
        headers = {
            'API-Key': self.api_key,
            'API-Sign': self._sign_payload(path, payload)
        }
        data = self._post(path, headers=headers, body=payload,
                          transformation=self._transform_open_orders,
                          model_class=KrakenOrder)
        return [order for order in data
                if order['symbol_pair'] == symbol_pair]

    def open_order(self, action, amount, symbol_pair, price, order_type):
        """
        Creates a new Order.

        :action:
            exchanges.ACTIONS choice
        :amount:
            Decimal, float, integer or string representing number value.
        :symbol_pair:
            currencies.SYMBOL_PAIRS choice
        :price:
            Decimal, float, integer or string representing number value.
        :order_type:
            exchanges.ORDER_TYPES choice
        """
        # validate arguments
        is_restricted_to_values(action, exchanges.ACTIONS)

        is_instance(amount, (Decimal, float, int, str))
        passes_test(amount, lambda x: Decimal(x))

        is_restricted_to_values(symbol_pair, currencies.SYMBOL_PAIRS)

        is_instance(price, (Decimal, float, int, str))
        passes_test(price, lambda x: isinstance(Decimal(x), Decimal))

        is_restricted_to_values(order_type, exchanges.ORDER_TYPES)

        path = '/0/private/AddOrder'
        userref = self.get_userref(symbol_pair)
        symbol_pair = self.SYMBOLS_MAPPING[symbol_pair]
        payload = {
            'pair': symbol_pair,
            'type': action,
            'ordertype': order_type,
            'price': Decimal(str(price)),
            'volume': Decimal(str(amount)),
            'leverage': 2,
            'nonce': int(1000 * time.time()),
            'userref': userref,
        }
        headers = {
            'API-Key': self.api_key,
            'API-Sign': self._sign_payload(path, payload)
        }
        return self._post(path, headers=headers, body=payload,
                          transformation=self._transform_new_order,
                          model_class=KrakenOrder)

    def cancel_order(self, order_id):
        path = '/0/private/CancelOrder'
        payload = {
            'nonce': int(1000 * time.time()),
            'txid': order_id
        }
        headers = {
            'API-Key': self.api_key,
            'API-Sign': self._sign_payload(path, payload)
        }
        return self._post(path, headers=headers, body=payload)

    def cancel_all_orders(self, symbol_pair):
        is_restricted_to_values(symbol_pair, currencies.SYMBOL_PAIRS)

        path = '/0/private/CancelOrder'
        payload = {
            'nonce': int(1000 * time.time()),
            'txid': self.get_userref(symbol_pair)
        }
        headers = {
            'API-Key': self.api_key,
            'API-Sign': self._sign_payload(path, payload)
        }
        return self._post(path, headers=headers, body=payload)

    def get_open_positions(self, symbol_pair):
        is_restricted_to_values(symbol_pair, currencies.SYMBOL_PAIRS)

        path = '/0/private/OpenPositions'
        payload = {
            'nonce': int(1000 * time.time()),
            'docalcs': True,
        }
        headers = {
            'API-Key': self.api_key,
            'API-Sign': self._sign_payload(path, payload)
        }
        positions = self._post(path, headers=headers, body=payload,
                               transformation=self._transform_open_positions,
                               model_class=KrakenPosition)
        return [pos for pos in positions
                if pos['symbol_pair'] == symbol_pair]

    def close_position(self, position_id, symbol_pair):
        is_restricted_to_values(symbol_pair, currencies.SYMBOL_PAIRS)

        positions = self.get_open_positions(symbol_pair)
        try:
            pos = [pos for pos in positions if pos.id == position_id][0]
        except IndexError:
            raise self.ERROR_CLASS('Could not find position with ID: "{}"'.format(position_id))

        # as we want to close the position,
        # we need to performe the opposite action to the given one.
        action = exchanges.SELL if pos.action == exchanges.BUY else exchanges.BUY

        return self.open_order(
            action, pos.amount, pos.symbol_pair, pos.price, exchanges.MARKET)

    def close_all_positions(self, symbol_pair):
        is_restricted_to_values(symbol_pair, currencies.SYMBOL_PAIRS)

        positions = self.get_open_positions(symbol_pair=symbol_pair)
        for pos in positions:
            self.close_position(pos.id, pos.symbol_pair)



import collections


def is_restricted_to_values(value, valid_values):
    assert isinstance(valid_values, collections.Iterable)
    if not value in valid_values:
        raise ValueError('Invalid "{}" value, expected any of: {}'
                         ''.format(value, valid_values))


def is_instance(value, instance_types):
    assert isinstance(instance_types, (type, collections.Iterable))
    if isinstance(instance_types, collections.Iterable):
        assert all([isinstance(elem, type) for elem in instance_types])
    if not isinstance(value, instance_types):
        raise ValueError('Invalid type "{}" for given value, expected any of: {}'
                         ''.format(type(value), instance_types))


def passes_test(value, test_func):
    assert callable(test_func)
    try:
        assert test_func(value)
    except AssertionError:
        raise ValueError("Given value didn't pass provided test")



from decimal import Decimal

from xchange.models.base import (Ticker, AccountBalance, OrderBook, Order,
                                 Position, contracts_to_crypto)


class OkexTicker(Ticker):
    """
    Original JSON response (transformed):
    {'date': '1506170137',
     'ticker': {
        'buy': 3740.31,
        'coin_vol': 0,
        'contract_id': 20171229012,
        'day_high': 0,
        'day_low': 0,
        'high': 3785.06,
        'last': 3740.31,
        'low': 3453.04,
        'sell': 3742.24,
        'unit_amount': 100,
        'vol': 3181196}}
    """
    def normalize_response(self, json_response):
        return {
            'ask': json_response['ticker']['sell'],
            'bid': json_response['ticker']['buy'],
            'low': json_response['ticker']['low'],
            'high': json_response['ticker']['high'],
            'last': json_response['ticker']['last'],
            'volume': json_response['ticker']['vol'],
        }


class OkexOrderBook(OrderBook):
    """
    Original JSON response format:
    {
        "asks": [
            # [price_in_btc, amount_in_contracts]
            [411.8, 6],
            [411.75, 11],
            [411.6, 22],
            [411.5, 9],
            [411.3, 16]
        ],
        "bids": [
            [410.65, 12],
            [410.64, 3],
            [410.19, 15],
            [410.18, 40],
            [410.09, 10]
        ]
    }
    """
    TICKER = None
    SYMBOL = None
    CONTRACT_UNIT_AMOUNTS = None

    def normalize_response(self, json_response):
        if all([self.TICKER, self.SYMBOL, self.CONTRACT_UNIT_AMOUNTS]):
            # if class atributes are provided, all contract amounts
            # are transformed to BTC amounts based on the ticker last price.
            last_price = self.TICKER.last
            unit_amount = self.CONTRACT_UNIT_AMOUNTS[self.SYMBOL]
            return {
                'asks': [(Decimal(str(doc[0])),
                          contracts_to_crypto(Decimal(str(doc[1])), last_price, unit_amount))
                         for doc in json_response['asks']],
                'bids': [(Decimal(str(doc[0])),
                          contracts_to_crypto(Decimal(str(doc[1])), last_price, unit_amount))
                         for doc in json_response['bids']],
            }
        else:
            return {
                'asks': [(Decimal(str(doc[0])), Decimal(str(doc[1])))
                         for doc in json_response['asks']],
                'bids': [(Decimal(str(doc[0])), Decimal(str(doc[1])))
                         for doc in json_response['bids']],
            }


class OkexAccountBalance(AccountBalance):
    """
    Response items format:
    {
        'amount': 0.162,
        'currency': 'btc',
    }
    """
    def normalize_response(self, json_response):
        return {
            'symbol': json_response['currency'],
            'amount': json_response['amount']
        }


class OkexOrder(Order):
    ORDER_STATUS = {
        -1: 'closed',  # cancelled
        0: 'open',  # unfilled
        1: 'open',  # partially_filled
        2: 'closed',  # fully_filled
        4: 'open',  # cancel_requested
    }
    ORDER_TYPE = {
        1: 'buy',  # open_long
        2: 'sell',  # open_short
        3: 'buy',  # close_long
        4: 'sell',  # close_short
    }
    """
    Response items format (transformed):
    {'amount': 1,
     'contract_name': 'BTC1229',
     'create_date': 1506188792000,
     'deal_amount': 0,
     'fee': 0,
     'lever_rate': 20,
     'order_id': 10602289748,
     'price': 3000,
     'price_avg': 0,
     'status': 0,
     'symbol': 'btc_usd',
     'type': 1,
     'unit_amount': 100}
    """
    def normalize_response(self, json_response):
        order = {
            'id': json_response['order_id']
        }
        keys = ['type', 'amount', 'price', 'symbol', 'status']
        if all([k in json_response for k in keys]):
            order.update({
            'action': self.ORDER_TYPE[json_response['type']],
            'amount': json_response['amount'],
            'price': json_response['price'],
            'symbol_pair': json_response['symbol'],
            'type': 'limit',
            'status': self.ORDER_STATUS[json_response['status']]
        })
        return order


class OkexPosition(Position):
    """
    Response items format (transformed):
    [{'action': 'buy',
      'amount': 1,
      'id': None,
      'price': 4269.38,
      'profit_loss': -7.03e-06,
      'symbol_pair': 'btc_usd'},
     {'action': 'sell',
      'amount': 1,
      'id': None,
      'price': 4266.363325,
      'profit_loss': -5.432e-05,
      'symbol_pair': 'btc_usd'}]
    """
    def normalize_response(self, json_response):
        return {
            'id': json_response['id'],
            'action': json_response['action'],
            'amount': json_response['amount'],
            'price': json_response['price'],
            'symbol_pair': json_response['symbol_pair'],
            'profit_loss': json_response['profit_loss'],
        }



import re
from decimal import Decimal

import responses

from tests import BaseXchangeTestCase
from xchange.clients.base import BaseExchangeClient
from xchange.exceptions import BaseXchangeException


class BaseClientTestCase(BaseXchangeTestCase):
    def setUp(self):
        super(BaseClientTestCase, self).setUp()
        self.client = BaseExchangeClient('API_KEY', 'API_SECRET')
        self.client.BASE_API_URL = 'https://xchage-testing.url'
        self.client.ERROR_CLASS = BaseXchangeException

    def test_init_arguments(self):
        self.assertEqual(self.client.api_key, 'API_KEY')
        self.assertEqual(self.client.api_secret, 'API_SECRET')

    def test_client_interface(self):
        methods = [
            # internal
            '_get',
            '_post',
            '_request',

            # non-authenticated
            'get_ticker',
            'get_order_book',

            # authenticated
            'get_account_balance',
            'get_open_orders',
            'get_open_positions',
            'get_order_status',
            'open_order',
            'cancel_order',
            'cancel_all_orders',
            'close_position',
            'close_all_positions',
        ]
        for method in methods:
            self.assertTrue(getattr(self.client, method) is not None)

    def test_client_interface_not_implement(self):
        methods = [
            ('get_ticker', ('symbol_pair', )),
            ('get_order_book', ('symbol_pair', )),
            ('get_account_balance', ()),
            ('get_open_orders', ('symbol_pair', )),
            ('get_open_positions', ('symbol_pair', )),
            ('get_order_status', ('order_id', )),
            ('open_order', ('action', 'amount', 'symbol_pair', 'price', 'order_type')),
            ('cancel_order', ('order_id', )),
            ('cancel_all_orders', ('symbol_pair', )),
            ('close_position', ('position_id', 'symbol_pair', )),
            ('close_all_positions', ('symbol_pair', )),
        ]
        for method, args in methods:
            with self.assertRaises(NotImplementedError):
                getattr(self.client, method)(**{arg_name: None for arg_name in args})

    @responses.activate
    def test_get_request_raw_data(self):
        """Should return response data as plain JSON when no model_class is provided"""
        responses.add(
            method='GET',
            url='{}/test-get-method'.format(self.client.BASE_API_URL),
            json={'success': True, 'msg': 'All good.'},
            status=200,
            content_type='application/json')
        data = self.client._get(path='/test-get-method', model_class=None)
        self.assertEqual(data, {'msg': 'All good.', 'success': True})

    @responses.activate
    def test_get_server_error(self):
        """Should raise client ERROR_CLASS when status code is 500"""
        responses.add(
            method='GET',
            url='{}/test-get-method'.format(self.client.BASE_API_URL),
            json={'success': False, 'msg': 'Something went wrong.'},
            status=500,
            content_type='application/json')
        with self.assertRaisesRegexp(BaseXchangeException, 'Got 500 response with content:'):
            self.client._get(path='/test-get-method', model_class=None)

    @responses.activate
    def test_get_bad_request(self):
        """Should raise client ERROR_CLASS when status code is 400"""
        responses.add(
            method='GET',
            url='{}/test-get-method'.format(self.client.BASE_API_URL),
            json={'success': False, 'msg': 'Invalid request params.'},
            status=400,
            content_type='application/json')
        with self.assertRaisesRegexp(BaseXchangeException, 'Got 400 response with content:'):
            self.client._get(path='/test-get-method', model_class=None)

    @responses.activate
    def test_get_empty_response_content(self):
        """Should return None when the response content is empty"""
        responses.add(
            method='GET',
            url='{}/test-get-method'.format(self.client.BASE_API_URL),
            body='',
            status=200,
            content_type='text/html')
        data = self.client._get(path='/test-get-method', model_class=None)
        self.assertEqual(data, None)

    @responses.activate
    def test_get_invalid_json_response_content(self):
        """Should raise client ERROR_CLASS when response JSON is badly formatted"""
        responses.add(
            method='GET',
            url='{}/test-get-method'.format(self.client.BASE_API_URL),
            body='{"not-valid-json"}',
            status=200,
            content_type='application/json')
        with self.assertRaisesRegexp(BaseXchangeException, 'Could not decode JSON response'):
            self.client._get(path='/test-get-method', model_class=None)

    @responses.activate
    def test_get_valid_response_code_with_error_message(self):
        """Should raise client ERROR_CLASS when status_code is 200 but response content includes error message"""
        responses.add(
            method='GET',
            url='{}/test-get-method'.format(self.client.BASE_API_URL),
            json={'error': 'Something went wrong.'},
            status=200,
            content_type='application/json')
        with self.assertRaisesRegexp(BaseXchangeException, 'Something went wrong'):
            self.client._get(path='/test-get-method', model_class=None)

    @responses.activate
    def test_get_apply_transformation_function(self):
        """Should apply transformation function to the response content if it's provided"""
        responses.add(
            method='GET',
            url='{}/test-get-method'.format(self.client.BASE_API_URL),
            json={'number': 100},
            status=200,
            content_type='application/json')
        transformation = lambda x: {'number': x['number']* 2}
        data = self.client._get(path='/test-get-method',
                                model_class=None, transformation=transformation)
        self.assertEqual(data, {'number': 200})

    @responses.activate
    def test_get_headers(self):
        """Should pass headers to the request when they are provided"""
        responses.add(
            method='GET',
            url='{}/test-get-method'.format(self.client.BASE_API_URL),
            json={'success': True, 'msg': 'All good.'},
            status=200,
            content_type='application/json')
        data = self.client._get(
            path='/test-get-method',
            model_class=None,
            headers={'X-CUSTOM-HEADER': 'xchange'}
        )
        self.assertEqual(data, {'msg': 'All good.', 'success': True})

    @responses.activate
    def test_get_custom_params(self):
        """Should add params to the URL when they are provided"""
        responses.add(
            method='GET',
            url=re.compile('{}/test-get-method\?custom_param=1'
                           ''.format(self.client.BASE_API_URL.replace('.', '\.'))),
            json={'success': True, 'msg': 'All good.'},
            status=200,
            content_type='application/json',
            match_querystring=True)
        data = self.client._get(
            path='/test-get-method',
            model_class=None,
            params={'custom_param': 1}
        )
        self.assertEqual(data, {'msg': 'All good.', 'success': True})

    @responses.activate
    def test_post_request(self):
        """Should return response as plain JSON when POSTing with no model_class"""
        responses.add(
            method='POST',
            url='{}/test-post-method'.format(self.client.BASE_API_URL),
            json={'success': True, 'msg': 'All good.'},
            status=200,
            content_type='application/json')
        data = self.client._post(
            path='/test-post-method',
            model_class=None)
        self.assertEqual(data, {'msg': 'All good.', 'success': True})

    @responses.activate
    def test_post_headers(self):
        """Should pass headers to the POST request when provided"""
        responses.add(
            method='POST',
            url='{}/test-post-method'.format(self.client.BASE_API_URL),
            json={'success': True, 'msg': 'All good.'},
            status=200,
            content_type='application/json')
        data = self.client._post(
            path='/test-post-method',
            model_class=None,
            headers={'X-CUSTOM-HEADER': 'xchange'}
        )
        self.assertEqual(data, {'msg': 'All good.', 'success': True})

    @responses.activate
    def test_post_custom_params(self):
        """Should pass params to the POST request when provided"""
        responses.add(
            method='POST',
            url=re.compile('{}/test-post-method\?custom_param=1'
                           ''.format(self.client.BASE_API_URL.replace('.', '\.'))),
            json={'success': True, 'msg': 'All good.'},
            status=200,
            content_type='application/json')
        data = self.client._post(
            path='/test-post-method',
            model_class=None,
            params={'custom_param': 1}
        )
        self.assertEqual(data, {'msg': 'All good.', 'success': True})

    @responses.activate
    def test_post_body(self):
        """Should pass body to the POST request if provided"""
        responses.add(
            method='POST',
            url='{}/test-post-method'.format(self.client.BASE_API_URL),
            json={'success': True, 'msg': 'All good.'},
            status=200,
            content_type='application/json')
        data = self.client._post(
            path='/test-post-method',
            model_class=None,
            body={'send-this': 'as-body'}
        )
        self.assertEqual(data, {'msg': 'All good.', 'success': True})

    @responses.activate
    def test_post_all_params_together(self):
        """Should pass all valid params to the POST request when they are provided"""
        responses.add(
            method='POST',
            url='{}/test-post-method'.format(self.client.BASE_API_URL),
            json={'success': True, 'msg': 'All good.'},
            status=200,
            content_type='application/json')
        data = self.client._post(
            path='/test-post-method',
            model_class=None,
            params={'custom_param': 1},
            body={'send-this': 'as-body'},
            headers={'X-CUSTOM-HEADER': 'xchange'}
        )
        self.assertEqual(data, {'msg': 'All good.', 'success': True})



from decimal import Decimal

from xchange import exceptions
from xchange.constants import exchanges
from xchange.validators import is_restricted_to_values, is_instance, passes_test


def worst_order_price(action, order_book, amount):
    """
    Calculates the worst used order price in given `order_book` to fulfill
    an operation (sell/buy) of the given `amount`.

    @params:
        * action: exchanges.ACTIONS choice
        * order_book: models.base.OrderBook instance or subclass
        * amount: Decimal or valid numeric argument representing volume to operate.

    @returns:
        a Decimal object representing the worst order price.
    """
    # validate arguments
    is_restricted_to_values(action, exchanges.ACTIONS)

    is_instance(amount, (Decimal, float, int, str))
    passes_test(amount, lambda x: Decimal(x))

    order_list = order_book.asks if action == exchanges.BUY else order_book.bids

    total_market_depth = sum([t[1] for t in order_list])
    if amount > total_market_depth:
        raise exceptions.InsufficientMarketDepth(
            'Not enough depth in OrderBook to {} {} volume'.format(action, amount))

    if action == exchanges.BUY:
        # when checking the "asks" list, we want to
        # iterate orders from cheapest to highest
        order_list = list(reversed(order_list))

    accum = Decimal('0')
    for price_tuple in order_list:
        price, volume = price_tuple
        accum += volume
        if accum >= amount:
            return price


def volume_weighted_average_price(action, order_book, amount):
    """
    Calculates the weighted average price of an operation (sell/buy)
    for a given `amount` and `order_book`.

    @params:
        * action: exchanges.ACTIONS choice
        * order_book: models.base.OrderBook instance or subclass
        * amount: Decimal or valid numeric argument representing volume to operate.

    @returns:
        a Decimal object representing the weighted average price.
    """
    # validate arguments
    is_restricted_to_values(action, exchanges.ACTIONS)

    is_instance(amount, (Decimal, float, int, str))
    passes_test(amount, lambda x: Decimal(x))

    order_list = order_book.asks if action == exchanges.BUY else order_book.bids

    total_market_depth = sum([t[1] for t in order_list])
    if amount > total_market_depth:
        raise exceptions.InsufficientMarketDepth(
            'Not enough depth in OrderBook to {} {} volume'.format(action, amount))

    if action == exchanges.BUY:
        # when checking the "asks" list, we want to
        # iterate orders from cheapest to highest
        order_list = list(reversed(order_list))
    accum = Decimal(0.0)

    # most of the times last used order in the orderbook
    # is partially used. we need to know which was the portion
    # used from that order to calculate the weighted average
    rest = amount

    for index, price_tuple in enumerate(order_list):
        volume = price_tuple[1]
        accum += volume
        if accum >= amount:
            break
        rest -= volume

    # get the sub list representing only the necessary
    # orders to fulfill the amount amount
    sub_list = order_list[:index + 1]

    # change the last used order in the list, to just
    # the needed rest amount
    sub_list[-1] = (sub_list[-1][0], Decimal(rest))

    sub_list_amounts = [t[1] for t in sub_list]
    return sum(x * y for x, y in sub_list) / sum(sub_list_amounts)




class BaseXchangeException(Exception):
    pass


class InvalidSymbolException(BaseXchangeException):
    pass


class InvalidSymbolPairException(BaseXchangeException):
    pass


class InvalidActionException(BaseXchangeException):
    pass


class InvalidAmountException(BaseXchangeException):
    pass


class InvalidPriceException(BaseXchangeException):
    pass


class InvalidOrderTypeException(BaseXchangeException):
    pass


class InvalidAmountContractsException(BaseXchangeException):
    pass


class InsufficientMarketDepth(BaseXchangeException):
    pass


class TimeoutException(BaseXchangeException):
    pass


# exchange exceptions
class BitfinexException(BaseXchangeException):
    pass


class OkexException(BaseXchangeException):
    pass


class KrakenException(BaseXchangeException):
    pass



import re
import responses
from decimal import Decimal

from tests import BaseXchangeTestCase
from tests.fixtures import okex
from xchange.exceptions import *
from xchange.factories import ExchangeClientFactory
from xchange.constants import exchanges, currencies
from xchange.models.okex import (
    OkexTicker, OkexOrderBook, OkexAccountBalance, OkexOrder)


class BaseOkexClientTestCase(BaseXchangeTestCase):
    def setUp(self):
        super(BaseOkexClientTestCase, self).setUp()
        self.ClientClass = ExchangeClientFactory.get_client(exchanges.OKEX)
        self.client = self.ClientClass('API_KEY', 'API_SECRET')

        for fixture in okex.FIXTURE_RESPONSES:
            responses.add(
                method=fixture['method'],
                url=re.compile(fixture['url_regex']),
                json=fixture['json'],
                status=fixture['status'],
                content_type=fixture['content_type'],
                match_querystring=True)


class OkexClientTickerTestCase(BaseOkexClientTestCase):

    @responses.activate
    def test_get_ticker(self):
        ticker = self.client.get_ticker(currencies.BTC_USD)
        expected = {
            'ask': Decimal('9314.66'),
            'bid': Decimal('100.51'),
            'high': Decimal('9480'),
            'last': Decimal('9312.34'),
            'low': Decimal('8800'),
            'volume': Decimal('16185076.0')
        }
        self.assertEqual(ticker, expected)
        self.assertEqual(type(ticker), OkexTicker)


class OkexClientOrderBookTestCase(BaseOkexClientTestCase):

    @responses.activate
    def test_get_order_book(self):
        order_book = self.client.get_order_book(currencies.BTC_USD)
        expected = {
            'asks': [
                (Decimal('9315.49'), Decimal('0.05369219766460417038037700515')),
                (Decimal('9315.47'), Decimal('0.8268598440349042238578058793')),
                (Decimal('9315.45'), Decimal('0.5261835371131208697276946505')),
                (Decimal('9315.06'), Decimal('0.1073843953292083407607540103')),
                (Decimal('9314.78'), Decimal('0.1073843953292083407607540103'))
            ],
            'bids': [
                (Decimal('9314.58'), Decimal('1.234920546285895918748671118')),
                (Decimal('9313.3'), Decimal('0.8805520416995083942381828845')),
                (Decimal('9313.04'), Decimal('0.9986748765616375690750122958')),
                (Decimal('9311.73'), Decimal('0.5369219766460417038037700515')),
                (Decimal('9311.42'), Decimal('0.6872601301069333808688256659')),
                (Decimal('9310.96'), Decimal('0.09664595579628750668467860927'))
            ]
        }
        self.assertEqual(order_book, expected)
        self.assertEqual(type(order_book), OkexOrderBook)


class OkexClientAccountBalanceTestCase(BaseOkexClientTestCase):

    @responses.activate
    def test_get_account_balance(self):
        balance = self.client.get_account_balance()
        expected = [
            {'amount': Decimal('0.01645887'), 'symbol': 'btc'},
            {'amount': Decimal('0'), 'symbol': 'ltc'}
        ]
        self.assertEqual(
            sorted(balance, key=lambda x: x['symbol']),
            sorted(expected, key=lambda x: x['symbol']))
        self.assertEqual(type(balance), list)
        for obj in balance:
            self.assertEqual(type(obj), OkexAccountBalance)

    @responses.activate
    def test_get_account_balance_symbol_pair(self):
        balance = self.client.get_account_balance(currencies.BTC)
        expected = {'amount': Decimal('0.01645887'), 'symbol': 'btc'}
        self.assertEqual(balance, expected)
        self.assertEqual(type(balance), OkexAccountBalance)


class OkexGetOpenOrdersTestCase(BaseOkexClientTestCase):

    @responses.activate
    def test_get_open_orders(self):
        fixture = {
            'orders': [
                {
                    'amount': 1,
                    'contract_name': 'BTC0929',
                    'create_date': 1503614569000,
                    'deal_amount': 0,
                    'fee': 0,
                    'lever_rate': 10,
                    'order_id': 8934112485,
                    'price': 5000,
                    'price_avg': 0,
                    'status': 0,
                    'symbol': 'btc_usd',
                    'type': 2,
                    'unit_amount': 100
                }
            ],
            'result': True
        }
        responses.add(
            method='POST',
            url=re.compile('https://www\.okex\.com/api/v1/future_order_info\.do'),
            json=fixture,
            status=200,
            content_type='application/json')
        open_orders = self.client.get_open_orders(currencies.BTC_USD)
        expected = [
            {
                'id': '8934112485',
                'action': 'sell',
                'amount': Decimal('1'),
                'price': Decimal('5000'),
                'status': 'open',
                'symbol_pair': 'btc_usd',
                'type': 'limit'
            }
        ]
        self.assertEqual(type(open_orders), list)
        for obj in open_orders:
            self.assertEqual(type(obj), OkexOrder)
        self.assertEqual(open_orders, expected)

    @responses.activate
    def test_get_open_orders_filters_by_symbol_pair(self):
        fixture = {
            'orders': [
                {
                    'amount': 1,
                    'contract_name': 'BTC0929',
                    'create_date': 1503614569000,
                    'deal_amount': 0,
                    'fee': 0,
                    'lever_rate': 10,
                    'order_id': 8934112485,
                    'price': 5000,
                    'price_avg': 0,
                    'status': 0,
                    'symbol': 'btc_usd',
                    'type': 2,
                    'unit_amount': 100
                },
                {
                    'amount': 1,
                    'contract_name': 'ETH0929',
                    'create_date': 1503614569000,
                    'deal_amount': 0,
                    'fee': 0,
                    'lever_rate': 10,
                    'order_id': 8934112485,
                    'price': 5000,
                    'price_avg': 0,
                    'status': 0,
                    'symbol': 'eth_usd',
                    'type': 2,
                    'unit_amount': 100
                },
            ],
            'result': True
        }
        responses.add(
            method='POST',
            url=re.compile('https://www\.okex\.com/api/v1/future_order_info\.do'),
            json=fixture,
            status=200,
            content_type='application/json')
        open_orders = self.client.get_open_orders(currencies.ETH_USD)
        expected = [
            {
                'id': '8934112485',
                'action': 'sell',
                'amount': Decimal('1'),
                'price': Decimal('5000'),
                'status': 'open',
                'symbol_pair': 'eth_usd',
                'type': 'limit'
            }
        ]
        self.assertEqual(type(open_orders), list)
        self.assertEqual(len(open_orders), 1)
        for obj in open_orders:
            self.assertEqual(type(obj), OkexOrder)
        self.assertEqual(open_orders, expected)

    @responses.activate
    def test_get_open_orders_no_open_orders(self):
        fixture = {'orders': [], 'result': True}
        responses.add(
            method='POST',
            url=re.compile('https://www\.okex\.com/api/v1/future_order_info\.do'),
            json=fixture,
            status=200,
            content_type='application/json')
        open_orders = self.client.get_open_orders(currencies.BTC_USD)
        self.assertEqual(type(open_orders), list)
        self.assertEqual(len(open_orders), 0)
        self.assertEqual(open_orders, [])


class OkexOpenOrderTestCase(BaseOkexClientTestCase):

    @responses.activate
    def test_open_order(self):
        fixture = {'order_id': 8931546905, 'result': True}
        responses.add(
            method='POST',
            url=re.compile('https://www\.okex\.com/api/v1/future_trade\.do'),
            json=fixture,
            status=200,
            content_type='application/json')
        order = self.client.open_order(
            action=exchanges.SELL,
            amount='0.01209215',
            symbol_pair=currencies.BTC_USD,
            price='4118.0',
            order_type=exchanges.LIMIT)
        expected = {
            'id': '8931546905'
        }
        self.assertEqual(order, expected)
        self.assertEqual(type(order), OkexOrder)

    @responses.activate
    def test_open_order_amount_in_contracts(self):
        fixture = {'order_id': 8931458965, 'result': True}
        responses.add(
            method='POST',
            url=re.compile('https://www\.okex\.com/api/v1/future_trade\.do'),
            json=fixture,
            status=200,
            content_type='application/json')
        order = self.client.open_order(
            action=exchanges.SELL,
            amount='2',
            amount_in_contracts=True,
            symbol_pair=currencies.BTC_USD,
            price='4118.0',
            order_type=exchanges.LIMIT)
        expected = {
            'id': '8931458965'
        }
        self.assertEqual(order, expected)
        self.assertEqual(type(order), OkexOrder)

    @responses.activate
    def test_open_order_amount_in_contracts_too_small(self):
        fixture = {'order_id': 8931458965, 'result': True}
        responses.add(
            method='POST',
            url=re.compile('https://www\.okex\.com/api/v1/future_trade\.do'),
            json=fixture,
            status=200,
            content_type='application/json')
        with self.assertRaisesRegexp(ValueError,
                                     "Given value didn't pass provided test"):
            self.client.open_order(
                action=exchanges.SELL,
                amount='0.021',  # must be greater or equal to 1
                amount_in_contracts=True,
                symbol_pair=currencies.BTC_USD,
                price='4118.0',
                order_type=exchanges.LIMIT)


class OkexCancelOrderTestCase(BaseOkexClientTestCase):

    @responses.activate
    def test_cancel_order(self):
        fixture = {
            'orders': [
                {
                    'order_id': 8934112485,
                    'amount': 1,
                    'contract_name': 'BTC0929',
                    'create_date': 1503614569000,
                    'deal_amount': 0,
                    'fee': 0,
                    'lever_rate': 10,
                    'price': 5000,
                    'price_avg': 0,
                    'status': 0,
                    'symbol': 'btc_usd',
                    'type': 2,
                    'unit_amount': 100
                }
            ],
            'result': True
        }
        responses.add(
            method='POST',
            url=re.compile('https://www\.okex\.com/api/v1/future_order_info\.do'),
            json=fixture,
            status=200,
            content_type='application/json')
        fixture = {'order_id': '8934112485', 'result': True}
        responses.add(
            method='POST',
            url=re.compile('https://www\.okex\.com/api/v1/future_cancel\.do'),
            json=fixture,
            status=200,
            content_type='application/json')
        response = self.client.cancel_order('8934112485')
        self.assertEqual(response, fixture)

    @responses.activate
    def test_cancel_order_id_not_found(self):
        fixture = {
            'orders': [],
            'result': True
        }
        responses.add(
            method='POST',
            url=re.compile('https://www\.okex\.com/api/v1/future_order_info\.do'),
            json=fixture,
            status=200,
            content_type='application/json')
        with self.assertRaisesRegexp(ValueError, 'Could not find order with ID "8934112485"'):
            self.client.cancel_order('8934112485')

    @responses.activate
    def test_cancel_all_orders(self):
        fixture = {
            'orders': [
                {
                    'order_id': 8934112485,
                    'amount': 1,
                    'contract_name': 'BTC0929',
                    'create_date': 1503614569000,
                    'deal_amount': 0,
                    'fee': 0,
                    'lever_rate': 10,
                    'price': 5000,
                    'price_avg': 0,
                    'status': 0,
                    'symbol': 'btc_usd',
                    'type': 2,
                    'unit_amount': 100
                },
                {
                    'order_id': 8913469485,
                    'amount': 1,
                    'contract_name': 'BTC0929',
                    'create_date': 1503254699000,
                    'deal_amount': 0,
                    'fee': 0,
                    'lever_rate': 10,
                    'price': 5000,
                    'price_avg': 0,
                    'status': 0,
                    'symbol': 'btc_usd',
                    'type': 2,
                    'unit_amount': 100
                },
            ],
            'result': True
        }
        responses.add(
            method='POST',
            url=re.compile('https://www\.okex\.com/api/v1/future_order_info\.do'),
            json=fixture,
            status=200,
            content_type='application/json')
        fixture = {'order_id': '8934112485,8913469485', 'result': True}
        responses.add(
            method='POST',
            url=re.compile('https://www\.okex\.com/api/v1/future_cancel\.do'),
            json=fixture,
            status=200,
            content_type='application/json')
        response = self.client.cancel_all_orders(currencies.BTC_USD)
        self.assertEqual(response, fixture)


class OkexGetOpenPositionsTestCase(BaseOkexClientTestCase):

    @responses.activate
    def test_get_open_positions(self):
        fixture = {
            'force_liqu_price': '10,604.11',
            'holding': [{
                'buy_amount': 0,
                'buy_available': 0,
                'buy_price_avg': 3348,
                'buy_price_cost': 3348,
                'buy_profit_real': 0.00052895,
                'contract_id': 20171222465,
                'contract_type': 'quarter',
                'create_date': 1505469260000,
                'lever_rate': 20,
                'sell_amount': 1,
                'sell_available': 1,
                'sell_price_avg': 3896.13,
                'sell_price_cost': 3896.13,
                'sell_profit_real': -0.1,
                'symbol': 'btc_usd'
            }],
            'result': True
        }
        responses.add(
            method='POST',
            url=re.compile('https://www\.okex\.com/api/v1/future_position\.do'),
            json=fixture,
            status=200,
            content_type='application/json')
        positions = self.client.get_open_positions(currencies.BTC_USD)
        expected = [
            {
                'action': 'sell',
                'amount': Decimal('1'),
                'id': 'None',
                'price': Decimal('3896.13'),
                'profit_loss': Decimal('-0.1'),
                'symbol_pair': 'btc_usd'
            }
        ]
        self.assertEqual(positions, expected)

    @responses.activate
    def test_get_open_positions_no_positions(self):
        fixture = {
            'force_liqu_price': '0.00',
            'holding': [{
                'buy_amount': 0,
                'buy_available': 0,
                'buy_price_avg': 3348,
                'buy_price_cost': 3348,
                'buy_profit_real': 0.00052895,
                'contract_id': 20171453212,
                'contract_type': 'quarter',
                'create_date': 1505612590000,
                'lever_rate': 20,
                'sell_amount': 0,
                'sell_available': 0,
                'sell_price_avg': 3896.13,
                'sell_price_cost': 3896.13,
                'sell_profit_real': 0.00010698,
                'symbol': 'btc_usd'
            }],
            'result': True
        }
        responses.add(
            method='POST',
            url=re.compile('https://www\.okex\.com/api/v1/future_position\.do'),
            json=fixture,
            status=200,
            content_type='application/json')
        positions = self.client.get_open_positions(currencies.BTC_USD)
        expected = []
        self.assertEqual(positions, expected)


class OkexClosePositionTestCase(BaseOkexClientTestCase):

    @responses.activate
    def test_close_position(self):
        with self.assertRaises(NotImplementedError):
            self.client.close_position('some-id', currencies.ETH_USD)

    @responses.activate
    def test_close_all_positions(self):
        fixture = {
            'force_liqu_price': '10,604.11',
            'holding': [{
                'buy_amount': 0,
                'buy_available': 0,
                'buy_price_avg': 3348,
                'buy_price_cost': 3348,
                'buy_profit_real': 0.00052895,
                'contract_id': 20171222465,
                'contract_type': 'quarter',
                'create_date': 1505469260000,
                'lever_rate': 20,
                'sell_amount': 1,
                'sell_available': 1,
                'sell_price_avg': 3896.13,
                'sell_price_cost': 3896.13,
                'sell_profit_real': -0.1,
                'symbol': 'btc_usd'
            }],
            'result': True
        }
        responses.add(
            method='POST',
            url=re.compile('https://www\.okex\.com/api/v1/future_position\.do'),
            json=fixture,
            status=200,
            content_type='application/json')
        fixture = {'order_id': 8945328905, 'result': True}
        responses.add(
            method='POST',
            url=re.compile('https://www\.okex\.com/api/v1/future_trade\.do'),
            json=fixture,
            status=200,
            content_type='application/json')
        self.client.close_all_positions(currencies.BTC_USD)



import re
from decimal import Decimal

import responses

from tests import BaseXchangeTestCase
from xchange.factories import ExchangeClientFactory
from xchange.constants import exchanges, currencies
from xchange.exceptions import KrakenException, InvalidSymbolPairException
from xchange.models.kraken import (
    KrakenTicker, KrakenOrderBook, KrakenAccountBalance, KrakenOrder,
    KrakenOrder, KrakenPosition)


class BaseKrakenClientTestCase(BaseXchangeTestCase):
    def setUp(self):
        super(BaseKrakenClientTestCase, self).setUp()
        self.ClientClass = ExchangeClientFactory.get_client(exchanges.KRAKEN)
        self.client = self.ClientClass('a' * 56, 'b' * 88)


class KrakenClientTickerTestCase(BaseKrakenClientTestCase):
    @responses.activate
    def test_get_ticker(self):
        fixture = {
            'error': [],
            'result': {
                'XETHZUSD': {
                    'a': ['775.97000', '8', '8.000'],
                    'b': ['774.04000', '2', '2.000'],
                    'c': ['774.23000', '0.00000032'],
                    'h': ['830.00000', '830.00000'],
                    'l': ['751.11000', '751.11000'],
                    'o': '812.50000',
                    'p': ['785.05276', '787.06592'],
                    't': [17253, 19730],
                    'v': ['41641.65603874', '46704.54167467']
                }
            }
        }
        responses.add(
            method='GET',
            url=re.compile('https://api.kraken.com/0/public/Ticker'),
            json=fixture,
            status=200,
            content_type='application/json')
        ticker = self.client.get_ticker(currencies.BTC_USD)
        expected = {
            'ask': Decimal('775.97000'),
            'bid': Decimal('774.04000'),
            'low': Decimal('751.11000'),
            'high': Decimal('830.00000'),
            'last': Decimal('774.23000'),
            'volume': Decimal('41641.65603874')
        }
        self.assertEqual(ticker, expected)
        self.assertEqual(type(ticker), KrakenTicker)

    @responses.activate
    def test_get_ticker_invalid_symbol_pair(self):
        with self.assertRaisesRegexp(ValueError,
                                     'Invalid "usd_usd" value, expected any of'):
            self.client.get_ticker('usd_usd')


class KrakenClientOrderBookTestCase(BaseKrakenClientTestCase):
    @responses.activate
    def test_get_order_book(self):
        fixture = {
            'error': [],
            'result': {
                'XETHZUSD': {
                    'asks': [
                        ['775.78000', '4.798', 1525637947],
                        ['775.82000', '4.398', 1525637948],
                        ['775.91000', '5.000', 1525637926],
                        ['775.93000', '0.579', 1525637923],
                        ['775.95000', '5.000', 1525637927]],
                    'bids': [
                        ['774.45000', '0.167', 1525637949],
                        ['773.73000', '2.699', 1525637947],
                        ['773.72000', '1.495', 1525637945],
                        ['773.70000', '12.808', 1525637947],
                        ['773.62000', '103.411', 1525637846]
                    ]
                }
            }
        }
        responses.add(
            method='GET',
            url=re.compile('https://api.kraken.com/0/public/Depth'),
            json=fixture,
            status=200,
            content_type='application/json')
        order_book = self.client.get_order_book(currencies.BTC_USD)
        expected = {
            'asks': [
                (Decimal('775.95000'), Decimal('5.000')),
                (Decimal('775.93000'), Decimal('0.579')),
                (Decimal('775.91000'), Decimal('5.000')),
                (Decimal('775.82000'), Decimal('4.398')),
                (Decimal('775.78000'), Decimal('4.798'))
            ],
            'bids': [
                (Decimal('774.45000'), Decimal('0.167')),
                (Decimal('773.73000'), Decimal('2.699')),
                (Decimal('773.72000'), Decimal('1.495')),
                (Decimal('773.70000'), Decimal('12.808')),
                (Decimal('773.62000'), Decimal('103.411'))
            ]
        }
        self.assertEqual(order_book, expected)
        self.assertEqual(type(order_book), KrakenOrderBook)


class KrakenClientAccountBalanceTestCase(BaseKrakenClientTestCase):
    @responses.activate
    def test_get_account_balance(self):
        fixture = {
            'error': [],
            'result': {
                'XETH': '0.0382000000',
                'XLTC': '0.0000050700',
                'XXBT': '0.0000043204',
                'ZUSD': '0.0807'
            }
        }
        responses.add(
            method='POST',
            url=re.compile('https://api.kraken.com/0/private/Balance'),
            json=fixture,
            status=200,
            content_type='application/json')
        balance = self.client.get_account_balance()
        expected = [
            {'amount': Decimal('0.0382000000'), 'symbol': 'eth'},
            {'amount': Decimal('0.0000050700'), 'symbol': 'ltc'},
            {'amount': Decimal('0.0000043204'), 'symbol': 'btc'},
            {'amount': Decimal('0.0807'), 'symbol': 'usd'}
        ]
        self.assertEqual(
            sorted(balance, key=lambda x: x['symbol']),
            sorted(expected, key=lambda x: x['symbol'])
        )

        self.assertEqual(type(balance), list)
        for obj in balance:
            self.assertEqual(type(obj), KrakenAccountBalance)

    @responses.activate
    def test_get_account_balance_symbol_pair(self):
        fixture = {
            'error': [],
            'result': {
                'XETH': '0.0382000000',
                'XLTC': '0.0000050700',
                'XXBT': '0.0000043204',
                'ZUSD': '0.0807'
            }
        }
        responses.add(
            method='POST',
            url=re.compile('https://api.kraken.com/0/private/Balance'),
            json=fixture,
            status=200,
            content_type='application/json')
        balance = self.client.get_account_balance(currencies.BTC)
        expected = {'symbol': 'btc', 'amount': Decimal('0.0000043204')}
        self.assertEqual(balance, expected)
        self.assertEqual(type(balance), KrakenAccountBalance)


class KrakenGetOpenOrdersTestCase(BaseKrakenClientTestCase):
    @responses.activate
    def test_get_open_orders(self):
        fixture =  {
            'error': [],
            'result': {
                'open': {
                    'ODF2C3-ABCDE-HUDKEN': {
                        'cost': '0.00000',
                        'descr': {
                            'close': '',
                            'leverage': '2:1',
                            'order': 'sell '
                            '0.02000000 '
                            'ETHUSD @ '
                            'limit 900.00 '
                            'with 2:1 '
                            'leverage',
                            'ordertype': 'limit',
                            'pair': 'ETHUSD',
                            'price': '900.00',
                            'price2': '0',
                            'type': 'sell'
                        },
                        'expiretm': 0,
                        'fee': '0.00000',
                        'limitprice': '0.00000',
                        'misc': '',
                        'oflags': 'fciq',
                        'opentm': 1525638064.8332,
                        'price': '0.00000',
                        'refid': None,
                        'starttm': 0,
                        'status': 'open',
                        'stopprice': '0.00000',
                        'userref': 7886259,
                        'vol': '0.02000000',
                        'vol_exec': '0.00000000'
                    }
                }
            }
        }
        responses.add(
            method='POST',
            url=re.compile('https://api.kraken.com/0/private/OpenOrders'),
            json=fixture,
            status=200,
            content_type='application/json')
        open_orders = self.client.get_open_orders(currencies.ETH_USD)
        expected =  [
            {
                'id': 'ODF2C3-ABCDE-HUDKEN',
                'action': 'sell',
                'amount': Decimal('0.02000000'),
                'price': Decimal('900.00'),
                'symbol_pair': 'eth_usd',
                'type': 'limit',
                'status': 'open'
            }
        ]
        self.assertEqual(type(open_orders), list)
        for obj in open_orders:
            self.assertEqual(type(obj), KrakenOrder)
        self.assertEqual(open_orders, expected)

    @responses.activate
    def test_get_open_orders_filters_by_symbol_pair(self):
        fixture =  {
            'error': [],
            'result': {
                'open': {
                    'ODF2C3-ABCDE-HUDKEN': {
                        'cost': '0.00000',
                        'descr': {
                            'close': '',
                            'leverage': '2:1',
                            'order': 'sell '
                            '0.02000000 '
                            'ETHUSD @ '
                            'limit 900.00 '
                            'with 2:1 '
                            'leverage',
                            'ordertype': 'limit',
                            'pair': 'ETHUSD',
                            'price': '900.00',
                            'price2': '0',
                            'type': 'sell'
                        },
                        'expiretm': 0,
                        'fee': '0.00000',
                        'limitprice': '0.00000',
                        'misc': '',
                        'oflags': 'fciq',
                        'opentm': 1525638064.8332,
                        'price': '0.00000',
                        'refid': None,
                        'starttm': 0,
                        'status': 'open',
                        'stopprice': '0.00000',
                        'userref': 7886259,
                        'vol': '0.02000000',
                        'vol_exec': '0.00000000'
                    },
                    'ODF2C3-FGHIJ-HUDKEN': {
                        'cost': '0.00000',
                        'descr': {
                            'close': '',
                            'leverage': '2:1',
                            'order': 'sell '
                            '0.02000000 '
                            'BTCUSD @ '
                            'limit 900.00 '
                            'with 2:1 '
                            'leverage',
                            'ordertype': 'limit',
                            'pair': 'BTCUSD',
                            'price': '900.00',
                            'price2': '0',
                            'type': 'sell'
                        },
                        'expiretm': 0,
                        'fee': '0.00000',
                        'limitprice': '0.00000',
                        'misc': '',
                        'oflags': 'fciq',
                        'opentm': 1525638064.8332,
                        'price': '0.00000',
                        'refid': None,
                        'starttm': 0,
                        'status': 'open',
                        'stopprice': '0.00000',
                        'userref': 7886259,
                        'vol': '0.02000000',
                        'vol_exec': '0.00000000'
                    },
                }
            }
        }
        responses.add(
            method='POST',
            url=re.compile('https://api.kraken.com/0/private/OpenOrders'),
            json=fixture,
            status=200,
            content_type='application/json')
        open_orders = self.client.get_open_orders(currencies.BTC_USD)
        expected =  [
            {
                'id': 'ODF2C3-FGHIJ-HUDKEN',
                'action': 'sell',
                'amount': Decimal('0.02000000'),
                'price': Decimal('900.00'),
                'symbol_pair': 'btc_usd',
                'type': 'limit',
                'status': 'open'
            }
        ]
        self.assertEqual(type(open_orders), list)
        self.assertEqual(len(open_orders), 1)
        for obj in open_orders:
            self.assertEqual(type(obj), KrakenOrder)
        self.assertEqual(open_orders, expected)

    @responses.activate
    def test_get_open_orders_empty_response(self):
        responses.add(
            method='POST',
            url=re.compile('https://api.kraken.com/0/private/OpenOrders'),
            json={'error': [], 'result': {'open': {}}},
            status=200,
            content_type='application/json')
        open_orders = self.client.get_open_orders(currencies.BTC_USD)
        self.assertEqual(type(open_orders), list)
        self.assertEqual(open_orders, [])


class KrakenOpenOrderTestCase(BaseKrakenClientTestCase):
    @responses.activate
    def test_open_order(self):
        fixture = {
            'error': [],
            'result': {
                'descr': {
                    'order': 'sell 0.02000000 ETHUSD @ limit 900.00 with 2:1 leverage'
                },
                'txid': ['ODF2C3-OVVBA-HUDKEN']
            }
        }
        responses.add(
            method='POST',
            url=re.compile('https://api.kraken.com/0/private/AddOrder'),
            json=fixture,
            status=200,
            content_type='application/json')
        order = self.client.open_order(
            action=exchanges.SELL,
            amount='0.01209215',
            symbol_pair=currencies.BTC_USD,
            price='4118.0',
            order_type=exchanges.LIMIT)
        expected = {'id': 'ODF2C3-OVVBA-HUDKEN'}
        self.assertEqual(order, expected)
        self.assertEqual(type(order), KrakenOrder)

    @responses.activate
    def test_open_order_bad_request(self):
        responses.add(
            method='POST',
            url=re.compile('https://api.kraken.com/0/private/AddOrder'),
            json={},
            status=400,
            content_type='application/json')
        with self.assertRaisesRegexp(KrakenException, 'Got 400 response with content:'):
            self.client.open_order(
                action=exchanges.SELL,
                amount='0.01209215',
                symbol_pair=currencies.BTC_USD,
                price='0.000000',  # won't work
                order_type=exchanges.LIMIT)


class KrakenCancelOrderTestCase(BaseKrakenClientTestCase):
    @responses.activate
    def test_cancel_order(self):
        order_id = 3439467895
        fixture = {'error': [], 'result': {'count': 1}}
        responses.add(
            method='POST',
            url=re.compile('https://api.kraken.com/0/private/CancelOrder'),
            json=fixture,
            status=200,
            content_type='application/json')
        response = self.client.cancel_order(order_id)
        self.assertEqual(response, fixture)

    @responses.activate
    def test_cancel_all_orders(self):
        fixture = {'error': [], 'result': {'count': 2}}
        responses.add(
            method='POST',
            url=re.compile('https://api.kraken.com/0/private/CancelOrder'),
            json=fixture,
            status=200,
            content_type='application/json')
        response = self.client.cancel_all_orders(currencies.BTC_USD)
        self.assertEqual(response, fixture)


class KrakenGetOpenPositionsTestCase(BaseKrakenClientTestCase):
    @responses.activate
    def test_get_open_positions(self):
        fixture = {
            'error': [],
            'result': {
                'TZESOY-ABCDE-V3K2B3': {
                    'cost': '15.48400',
                    'fee': '0.04336',
                    'margin': '7.74200',
                    'misc': '',
                    'net': '-0.0178',
                    'oflags': '',
                    'ordertxid': 'O4M6JE-W3LHA-66LKK4',
                    'ordertype': 'market',
                    'pair': 'XETHZUSD',
                    'posstatus': 'open',
                    'rollovertm': '1525653978',
                    'terms': '0.0200% per 4 hours',
                    'time': 1525639578.5486,
                    'type': 'sell',
                    'value': '15.50',
                    'vol': '0.02000000',
                    'vol_closed': '0.00000000'
                }
            }
        }
        responses.add(
            method='POST',
            url=re.compile('https://api.kraken.com/0/private/OpenPositions'),
            json=fixture,
            status=200,
            content_type='application/json')
        positions = self.client.get_open_positions(currencies.ETH_USD)
        expected = [
            {
                'action': 'sell',
                'amount': Decimal('0.02000000'),
                'id': 'TZESOY-ABCDE-V3K2B3',
                'price': Decimal('774.2'),
                'profit_loss': Decimal('-0.0178'),
                'symbol_pair': 'eth_usd'
            }
        ]
        self.assertEqual(positions, expected)
        self.assertEqual(len(positions), 1)
        self.assertEqual(type(positions), list)
        for obj in positions:
            self.assertEqual(type(obj), KrakenPosition)

    @responses.activate
    def test_get_open_positions_filters_symbol_pair(self):
        fixture = {
            'error': [],
            'result': {
                'TZESOY-ABCDE-V3K2B3': {
                    'cost': '15.48400',
                    'fee': '0.04336',
                    'margin': '7.74200',
                    'misc': '',
                    'net': '-0.0178',
                    'oflags': '',
                    'ordertxid': 'O4M6JE-W3LHA-66LKK4',
                    'ordertype': 'market',
                    'pair': 'XETHZUSD',
                    'posstatus': 'open',
                    'rollovertm': '1525653978',
                    'terms': '0.0200% per 4 hours',
                    'time': 1525639578.5486,
                    'type': 'sell',
                    'value': '15.50',
                    'vol': '0.02000000',
                    'vol_closed': '0.00000000'
                },
                'TZESOY-FGHIJ-V3K2B3': {
                    'cost': '15.48400',
                    'fee': '0.04336',
                    'margin': '7.74200',
                    'misc': '',
                    'net': '-0.0178',
                    'oflags': '',
                    'ordertxid': 'O4M6JE-AHUSY-66LKK4',
                    'ordertype': 'market',
                    'pair': 'XXBTZUSD',
                    'posstatus': 'open',
                    'rollovertm': '1525653978',
                    'terms': '0.0200% per 4 hours',
                    'time': 1525639578.5486,
                    'type': 'sell',
                    'value': '15.50',
                    'vol': '0.02000000',
                    'vol_closed': '0.00000000'
                },
            }
        }
        responses.add(
            method='POST',
            url=re.compile('https://api.kraken.com/0/private/OpenPositions'),
            json=fixture,
            status=200,
            content_type='application/json')
        positions = self.client.get_open_positions(currencies.ETH_USD)
        expected = [
            {
                'action': 'sell',
                'amount': Decimal('0.02000000'),
                'id': 'TZESOY-ABCDE-V3K2B3',
                'price': Decimal('774.2'),
                'profit_loss': Decimal('-0.0178'),
                'symbol_pair': 'eth_usd'
            }
        ]
        self.assertEqual(positions, expected)
        self.assertEqual(len(positions), 1)
        self.assertEqual(type(positions), list)
        for obj in positions:
            self.assertEqual(type(obj), KrakenPosition)

    @responses.activate
    def test_get_open_positions_no_open_position(self):
        responses.add(
            method='POST',
            url=re.compile('https://api.kraken.com/0/private/OpenPositions'),
            json={'error': [], 'result': {}},
            status=200,
            content_type='application/json')
        positions = self.client.get_open_positions(currencies.BTC_USD)
        self.assertEqual(positions, [])
        self.assertEqual(len(positions), 0)
        self.assertEqual(type(positions), list)


class KrakenClosePositionTestCase(BaseKrakenClientTestCase):
    @responses.activate
    def test_close_position(self):
        fixture = {
            'error': [],
            'result': {
                'TZESOY-ABCDE-V3K2B3': {
                    'cost': '15.48400',
                    'fee': '0.04336',
                    'margin': '7.74200',
                    'misc': '',
                    'net': '-0.0178',
                    'oflags': '',
                    'ordertxid': 'O4M6JE-W3LHA-66LKK4',
                    'ordertype': 'market',
                    'pair': 'XETHZUSD',
                    'posstatus': 'open',
                    'rollovertm': '1525653978',
                    'terms': '0.0200% per 4 hours',
                    'time': 1525639578.5486,
                    'type': 'sell',
                    'value': '15.50',
                    'vol': '0.02000000',
                    'vol_closed': '0.00000000'
                }
            }
        }
        responses.add(
            method='POST',
            url=re.compile('https://api.kraken.com/0/private/OpenPositions'),
            json=fixture,
            status=200,
            content_type='application/json')
        fixture = {
            'error': [],
            'result': {
                'descr': {
                    'order': 'sell 0.02000000 ETHUSD @ limit 900.00 with 2:1 leverage'
                },
                'txid': ['ODF2C3-QUIOA-HUDKEN']
            }
        }
        responses.add(
            method='POST',
            url=re.compile('https://api.kraken.com/0/private/AddOrder'),
            json=fixture,
            status=200,
            content_type='application/json')
        order = self.client.close_position(
            position_id='TZESOY-ABCDE-V3K2B3',
            symbol_pair=currencies.ETH_USD)
        expected = {'id': 'ODF2C3-QUIOA-HUDKEN'}
        self.assertEqual(order, expected)
        self.assertEqual(type(order), KrakenOrder)

    @responses.activate
    def test_close_position_id_not_found(self):
        fixture = {
            'error': [],
            'result': {
                'TZESOY-ABCDE-V3K2B3': {
                    'cost': '15.48400',
                    'fee': '0.04336',
                    'margin': '7.74200',
                    'misc': '',
                    'net': '-0.0178',
                    'oflags': '',
                    'ordertxid': 'O4M6JE-W3LHA-66LKK4',
                    'ordertype': 'market',
                    'pair': 'XETHZUSD',
                    'posstatus': 'open',
                    'rollovertm': '1525653978',
                    'terms': '0.0200% per 4 hours',
                    'time': 1525639578.5486,
                    'type': 'sell',
                    'value': '15.50',
                    'vol': '0.02000000',
                    'vol_closed': '0.00000000'
                }
            }
        }
        responses.add(
            method='POST',
            url=re.compile('https://api.kraken.com/0/private/OpenPositions'),
            json=fixture,
            status=200,
            content_type='application/json')
        with self.assertRaisesRegexp(KrakenException, 'Could not find position with ID: "WONT-BE-FOUND"'):
            order = self.client.close_position(
                position_id='WONT-BE-FOUND',
                symbol_pair=currencies.ETH_USD)

    @responses.activate
    def test_close_all_positions(self):
        fixture = {
            'error': [],
            'result': {
                'TZESOY-ABCDE-V3K2B3': {
                    'cost': '15.48400',
                    'fee': '0.04336',
                    'margin': '7.74200',
                    'misc': '',
                    'net': '-0.0178',
                    'oflags': '',
                    'ordertxid': 'O4M6JE-W3LHA-66LKK4',
                    'ordertype': 'market',
                    'pair': 'XETHZUSD',
                    'posstatus': 'open',
                    'rollovertm': '1525653978',
                    'terms': '0.0200% per 4 hours',
                    'time': 1525639578.5486,
                    'type': 'sell',
                    'value': '15.50',
                    'vol': '0.02000000',
                    'vol_closed': '0.00000000'
                }
            }
        }
        responses.add(
            method='POST',
            url=re.compile('https://api.kraken.com/0/private/OpenPositions'),
            json=fixture,
            status=200,
            content_type='application/json')
        fixture = {
            'error': [],
            'result': {
                'descr': {
                    'order': 'sell 0.02000000 ETHUSD @ limit 900.00 with 2:1 leverage'
                },
                'txid': ['ODF2C3-QUIOA-HUDKEN']
            }
        }
        responses.add(
            method='POST',
            url=re.compile('https://api.kraken.com/0/private/AddOrder'),
            json=fixture,
            status=200,
            content_type='application/json')
        self.client.close_all_positions(currencies.BTC_USD)



from xchange import exceptions
from xchange.constants import currencies
from xchange.models.utils import (
    as_decimal, sorted_list, restricted_to_values,
    normalized_symbol, normalized_symbol_pair,
    contracts_to_crypto, crypto_to_contracts
)


class BaseExchangeModel(dict):
    schema = {}

    def __init__(self, json_response):
        super(BaseExchangeModel, self).__init__()
        parsed_response = self.normalize_response(json_response)
        self.assign_dynamic_attributes(parsed_response)

    def __getattr__(self, key):
        try:
            return self[key]
        except KeyError:
            raise AttributeError(
                'Object {} has not attribute "{}"'
                ''.format(self.__class__.__name__, key))

    def normalize_response(self, json_response):
        """As this is the base class, don't perform any transformation"""
        return json_response

    def assign_dynamic_attributes(self, parsed_response):
        """
        Dynamically assign fields in the response as attributes
        of the Model.
        For each field, a formatting function is applied according
        to the provided "schema".
        """
        for field, value in parsed_response.items():
            func = self.schema.get(field)
            if not func:
                raise ValueError(
                    'Unknown field "{}" for class {}'
                    ''.format(field, self.__class__.__name__))
            self[field] = func(value)


class Ticker(BaseExchangeModel):
    schema = {
        'ask': as_decimal,
        'bid': as_decimal,
        'low': as_decimal,
        'high': as_decimal,
        'last': as_decimal,
        'volume': as_decimal,
    }


class OrderBook(BaseExchangeModel):
    """
    Normalized format:
    {
        "asks": [
            # (price_in_btc, amount_in_btc),
            (Decimal('4630.12300'), Decimal('0.014')),
            (Decimal('4620.23450'), Decimal('0.456')),
        ],
        "bids": [
            # (price_in_btc, amount_in_btc),
            (Decimal('4610.54856'), Decimal('0.078')),
            (Decimal('4600.78952'), Decimal('0.125')),
        ]
    }
    """
    schema = {
        'asks': sorted_list(key=lambda l: l[0], sorting_type='desc'),
        'bids': sorted_list(key=lambda l: l[0], sorting_type='desc'),
    }


class AccountBalance(BaseExchangeModel):
    schema = {
        'symbol': normalized_symbol,
        'amount': as_decimal,
    }


class Order(BaseExchangeModel):
    schema = {
        'id': str,
        'action': restricted_to_values(('sell', 'buy')),
        'amount': as_decimal,
        'price': as_decimal,
        'symbol_pair': normalized_symbol_pair,
        'type': restricted_to_values(('limit', 'market')),
        'status': restricted_to_values(('open', 'closed')),
    }


class Position(BaseExchangeModel):
    schema = {
        'id': str,
        'action': restricted_to_values(('sell', 'buy')),
        'amount': as_decimal,
        'price': as_decimal,
        'symbol_pair': normalized_symbol_pair,
        'profit_loss': as_decimal,
    }



import requests
from requests.adapters import HTTPAdapter
from decimal import Decimal
try:
    from urllib.parse import urlencode
except ImportError:
     from urllib import urlencode
try:
    from json.decoder import JSONDecodeError
except ImportError:
    JSONDecodeError = ValueError

from .. import exceptions


class BaseExchangeClient:

    def __init__(self, api_key, api_secret):
        self.api_key = api_key
        self.api_secret = api_secret

    def _get(self, path, headers=None, params=None,
             transformation=None, model_class=None, **kwargs):
        if params:
            path += '?' + urlencode(params)
        return self._request(
            'GET',
            path=path,
            headers=headers,
            model_class=model_class,
            transformation=transformation, **kwargs
        )

    def _post(self, path, headers=None, params=None, body=None,
              transformation=None, model_class=None, **kwargs):
        if params:
            path += '?' + urlencode(params)
        if body:
            body = urlencode(body)
        return self._request(
            'POST',
            path=path,
            body=body,
            headers=headers,
            model_class=model_class,
            transformation=transformation, **kwargs
        )

    def _request(self, method, path, headers=None, body=None,
                 transformation=None, model_class=None, timeout=3,
                 max_retries=0):
        request = requests.Request(
            method=method,
            url=self.BASE_API_URL + path,
            data=body)
        if headers:
            request.headers.update(headers)

        session = requests.Session()
        if max_retries:
            adapter = HTTPAdapter(max_retries=max_retries)
            session.mount('http://', adapter)
            session.mount('https://', adapter)

        try:
            response = session.send(
                request.prepare(),
                timeout=timeout)
        except requests.exceptions.ConnectTimeout:
            raise exceptions.TimeoutException()

        return self._process_response(
            response, model_class, transformation)

    def _process_response(self, response, model_class=None, transformation=None):
        # check for error-related status codes
        if response.status_code != requests.status_codes.codes.ok:
            raise self.ERROR_CLASS(
                'Got {} response with content: {}'
                ''.format(response.status_code, response.content)
            )

        # if response content is None, avoid data processing
        if not response.content:
            return

        # make sure it's a JSON valid response
        try:
            data = response.json()
        except JSONDecodeError as exc:
            raise self.ERROR_CLASS(
                'Could not decode JSON response, got: {}'.format(exc))

        if isinstance(data, dict):
            # some APIs return 200 status code, but include the error
            # detail as part of the response payload
            if (data.get('error') or
                    data.get('error_code') or
                    ('result' in data and data['result'] == False)):
                raise self.ERROR_CLASS(data)

        # if a transformation function was provided, replace the original
        # JSON response with the result of the transformation function
        if transformation:
            data = transformation(data)

        # when model_class is not provided, return the raw response data
        if not model_class:
            return data

        # create model instances using the response JSON data
        if isinstance(data, list):
            data = list(map(model_class, data))
        else:
            data = model_class(data)
        return data

    def _empty_account_balance(self, symbol):
        return {'symbol': symbol, 'amount': Decimal('0')}

    # public endpoints

    def get_ticker(self, symbol_pair, **kwargs):
        raise NotImplementedError

    def get_order_book(self, symbol_pair, **kwargs):
        raise NotImplementedError

    # authenticated endpoints

    def get_account_balance(self, symbol=None, **kwargs):
        raise NotImplementedError

    def get_open_orders(self, symbol_pair, **kwargs):
        raise NotImplementedError

    def get_open_positions(self, symbol_pair, **kwargs):
        raise NotImplementedError

    def get_order_status(self, order_id, **kwargs):
        raise NotImplementedError

    def open_order(self, action, amount, symbol_pair,
                   price, order_type, **kwargs):
        raise NotImplementedError

    def cancel_order(self, order_id, **kwargs):
        raise NotImplementedError

    def cancel_all_orders(self, symbol_pair, **kwargs):
        raise NotImplementedError

    def close_position(self, position_id, symbol_pair, **kwargs):
        raise NotImplementedError

    def close_all_positions(self, symbol_pair, **kwargs):
        raise NotImplementedError



import setuptools
import xchange


with open('README.md', 'r') as fh:
    long_description = fh.read()


def read_requirements(file_name):
    with open(file_name, 'r') as fp:
        return [l.strip() for l in fp if l.strip() and not l.startswith('-r')]


setuptools.setup(
    name='xchange',
    version=xchange.__version__,
    description=('Many cryptocurrency exchange APIs, a single and unified API client'),
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/martinzugnoni/xchange',
    download_url=(
        'https://github.com/martinzugnoni/xchange/tarball/{version}'.format(
            version=xchange.__version__)),
    author='Martin Zugnoni',
    author_email='martin.zugnoni@gmail.com',
    license='MIT',
    packages=setuptools.find_packages(),
    maintainer='Martin Zugnoni',
    install_requires=read_requirements('requirements/base.txt'),
    tests_require=read_requirements('requirements/dev.txt'),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)



"""
xchange: Many cryptocurrency exchange APIs, a single and unified API client.
~~~~~~~~~~~~~~~~~~~~~

:copyright: (c) 2018-2019 by Martin Zugnoni.
:license: MIT License, see LICENSE for more details.
"""

__title__ = 'xchange'
__version__ = '0.0.6'
__build__ = 0x00001
__author__ = 'Martin Zugnoni'
__license__ = 'MIT'
__copyright__ = 'Copyright 2018-2019 Martin Zugnoni'
